package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.GctcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GctcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gctcan Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface GctcanStructMapper extends IBaseStructMapper<GctcanStoreBo, GctcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GctcanStructMapper INSTANCE = Mappers.getMapper(GctcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GctcanStoreBo voTransferToBo(GctcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mapping(target = "gidgrp", ignore = true)
    @Mapping(target = "gcdgrp", ignore = true)
    void voUpdateToBo(GctcanStoreVo vo, @MappingTarget GctcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(GctcanStoreVo vo, @MappingTarget GctcanStoreBo bo) {
        bo.getGcdgrp().getRec().setClsdat(vo.getGcdgrp().getRec().getClsdat());
        bo.setRebkpflg(vo.getRebkpflg());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GctcanStoreVo boTransferToVo(GctcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GctcanStoreBo bo, @MappingTarget GctcanStoreVo vo);
}
