package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NitsetStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitsetStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NitsetStructMapper extends IBaseStructMapper<NitsetStoreBo, NitsetStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitsetStructMapper INSTANCE = Mappers.getMapper(NitsetStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitsetStoreBo voTransferToBo(NitsetStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "nidgrp",ignore = true)
    })
    void voUpdateToBo(NitsetStoreVo vo, @MappingTarget NitsetStoreBo bo);

    @AfterMapping
    default void afterUpdate(NitsetStoreVo vo, @MappingTarget NitsetStoreBo bo){
        bo.getNidgrp().getRec().setRptref(vo.getNidgrp().getRec().getRptref());
        bo.getNidgrp().getRec().setInvref(vo.getNidgrp().getRec().getInvref());
        bo.getNidgrp().getRec().setCmtflg(vo.getNidgrp().getRec().getCmtflg());
    }
    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitsetStoreVo boTransferToVo(NitsetStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitsetStoreBo bo, @MappingTarget NitsetStoreVo vo);
}
